/*
   This program tests the longest common subsequence function,
   lcs(a1,a2,n), where n is a command line parameter and
   a1, a2 are random permutations of the integers from 1 to n.
   The time it takes is (roughly) calculated.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>      // time()
#include <windows.h>   // needed for LARGE_INTEGER  and QueryPerformanceCounter()
#include "Timing.h"
#include "LCS.h"


/*
   This function will randomly permute the elements of the array
   between index low and high.
*/
void permutation(int* a, int low, int high)
{
   int i, upperBound, temp;

   //srand( time(NULL) );

   upperBound = high;
   // The array from indices low to upperBound is the "jar" to draw randomly
   // from, and the array from indices upperBound to high is the list of
   // numbers that have already been drawn out of the "jar".
   for (i = low; i <= high; i++)
   {
      // pick a slot in the "jar" part of the array
      int randomIndex = low + (rand() % (upperBound-(low-1)));

      // swap a[upperBound-1] with a[randomIndex]
      // (that is, move an item from the "jar" part of the array
      // to the list part of the array)
      temp = a[upperBound];
      a[upperBound] = a[randomIndex];
      a[randomIndex] = temp;

      // lower the upperbound
      upperBound--;  // lower the boundary of the "jar" part of the array
   }
}//permutation()


/*
   This function prints an array.
*/
void printTheArray(int* a, int low, int high)
{
   int i;
   printf("Here is the array.\n");
   for (i = low; i <= high; i++)
   {
      printf(" %*d", 2, a[i]);
   }
   printf("\n");
}//printTheArray()




int main(int argc, char** argv)
{
   LARGE_INTEGER timeStart, timeEnd;             // time in clock-ticks
   LARGE_INTEGER clockFrequency;                 // clock-ticks per second
   double elapsedTicks;
   TimingObject startTime, endTime, elapsedTime; // time in milli-seconds

   int length;
   int* integerArray1;
   int* integerArray2;
   int i;

   int n;
   if (argc > 1)
   {
      n = atoi( argv[1] );  // get parameter from command line
   }
   else
   {
      printf("Usage: lcs <n>");
      return 0;
   }

   integerArray1 = malloc(n*sizeof(int));  // array that will hold a permutation.
   if (integerArray1 == NULL)  // error check
   {
      fprintf(stderr, "lcs: not enough memory for array of size %d.\n", n);
      return -1;
   }

   integerArray2 = malloc(n*sizeof(int));  // array that will hold a permutation.
   if (integerArray2 == NULL)  // error check
   {
      fprintf(stderr, "lcs: not enough memory for array of size %d.\n", n);
      return -1;
   }

   // initialize the arrays with the numbers for 1 to n
   srand( time(NULL) );
   for (i = 0; i < n; i++)
   {
      integerArray1[i] = i+1;
      integerArray2[i] = i+1;
   }

   // the permutation() function will randomly permute
   // the contents of the arrays
   permutation(integerArray1, 0, n-1);
   permutation(integerArray2, 0, n-1);

   // if n is small enough, print out the arrays.
   if (n <= 30)
   {
      printTheArray(integerArray1, 0, n-1);
   }
   if (n <= 30)
   {
      printTheArray(integerArray2, 0, n-1);
   }

   // get number of clock ticks per second
   QueryPerformanceFrequency(&clockFrequency);

   // Compute the length of the
   // longest common subsequence
   // in the two suffled arrays.
   getThreadTime( &startTime );           // milli-second timer
   QueryPerformanceCounter( &timeStart ); // clock-tick timer
   length = lcs2( integerArray1, integerArray2, n, n );
   QueryPerformanceCounter( &timeEnd );
   getThreadTime( &endTime );

   // Compute the elapsed time.
   elapsedTicks = timeEnd.QuadPart - timeStart.QuadPart;
   computeElapsedTime( &startTime, &endTime, &elapsedTime );

   printf("\n");
   printf("lcs(a1, a2, %d) = %d,\n", n, length);
   printf("and it took\n");
   printf("\t%f seconds,\n", elapsedTicks/clockFrequency.QuadPart);
   printTimingObject( &elapsedTime );
   printf("to compute.\n");

   return 0;
}//main()
